_GROUPS = [
    {
        'code': '01',
        'name': 'زراعت و خدمات وابسته',
    },
    {
        'code': '02',
        'name': 'جنگلداري و ماهيگيري',
    },
    {
        'code': '10',
        'name': 'استخراج زغال سنگ',
    },
    {
        'code': '11',
        'name': 'استخراج نفت گاز و خدمات جنبي جز اکتشاف',
    },
    {
        'code': '13',
        'name': 'استخراج کانه هاي فلزي',
    },
    {
        'code': '14',
        'name': 'استخراج ساير معادن',
    },
    {
        'code': '15',
        'name': 'حذف شده- فرآورده‌هاي غذايي و آشاميدني',
    },
    {
        'code': '17',
        'name': 'منسوجات',
    },
    {
        'code': '19',
        'name': 'دباغي، پرداخت چرم و ساخت انواع پاپوش',
    },
    {
        'code': '20',
        'name': 'محصولات چوبي',
    },
    {
        'code': '21',
        'name': 'محصولات كاغذي',
    },
    {
        'code': '22',
        'name': 'انتشار، چاپ و تکثير',
    },
    {
        'code': '23',
        'name': 'فراورده هاي نفتي، كك و سوخت هسته اي',
    },
    {
        'code': '24',
        'name': 'حذف شده-مواد و محصولات شيميايي',
    },
    {
        'code': '25',
        'name': 'لاستيك و پلاستيك',
    },
    {
        'code': '26',
        'name': 'توليد محصولات كامپيوتري الكترونيكي ونوري',
    },
    {
        'code': '27',
        'name': 'فلزات اساسي',
    },
    {
        'code': '28',
        'name': 'ساخت محصولات فلزي',
    },
    {
        'code': '29',
        'name': 'ماشين آلات و تجهيزات',
    },
    {
        'code': '31',
        'name': 'ماشين آلات و دستگاه‌هاي برقي',
    },
    {
        'code': '32',
        'name': 'ساخت دستگاه‌ها و وسايل ارتباطي',
    },
    {
        'code': '33',
        'name': 'ابزارپزشکي، اپتيکي و اندازه‌گيري',
    },
    {
        'code': '34',
        'name': 'خودرو و ساخت قطعات',
    },
    {
        'code': '35',
        'name': 'ساير تجهيزات حمل و نقل',
    },
    {
        'code': '36',
        'name': 'مبلمان و مصنوعات ديگر',
    },
    {
        'code': '38',
        'name': 'قند و شكر',
    },
    {
        'code': '39',
        'name': 'شرکتهاي چند رشته اي صنعتي',
    },
    {
        'code': '40',
        'name': 'عرضه برق، گاز، بخاروآب گرم',
    },
    {
        'code': '41',
        'name': 'جمع آوري، تصفيه و توزيع آب',
    },
    {
        'code': '42',
        'name': 'محصولات غذايي و آشاميدني به جز قند و شكر',
    },
    {
        'code': '43',
        'name': 'مواد و محصولات دارويي',
    },
    {
        'code': '44',
        'name': 'محصولات شيميايي',
    },
    {
        'code': '45',
        'name': 'پيمانكاري صنعتي',
    },
    {
        'code': '46',
        'name': 'تجارت عمده فروشي به جز وسايل نقليه موتور',
    },
    {
        'code': '47',
        'name': 'خرده فروشي،باستثناي وسايل نقليه موتوري',
    },
    {
        'code': '49',
        'name': 'كاشي و سراميك',
    },
    {
        'code': '50',
        'name': 'تجارت عمده وخرده فروشي وسائط نقليه موتور',
    },
    {
        'code': '51',
        'name': 'حمل و نقل هوايي',
    },
    {
        'code': '52',
        'name': 'انبارداري و حمايت از فعاليتهاي حمل و نقل',
    },
    {
        'code': '53',
        'name': 'سيمان، آهك و گچ',
    },
    {
        'code': '54',
        'name': 'ساير محصولات كاني غيرفلزي',
    },
    {
        'code': '55',
        'name': 'هتل و رستوران',
    },
    {
        'code': '56',
        'name': 'سرمايه گذاريها',
    },
    {
        'code': '57',
        'name': 'بانكها و موسسات اعتباري',
    },
    {
        'code': '58',
        'name': 'ساير واسطه گريهاي مالي',
    },
    {
        'code': '59',
        'name': 'اوراق حق تقدم استفاده از تسهيلات مسكن',
    },
    {
        'code': '60',
        'name': 'حمل ونقل، انبارداري و ارتباطات',
    },
    {
        'code': '61',
        'name': 'حمل و نقل آبي',
    },
    {
        'code': '63',
        'name': 'فعاليت هاي پشتيباني و كمكي حمل و نقل',
    },
    {
        'code': '64',
        'name': 'مخابرات',
    },
    {
        'code': '65',
        'name': 'واسطه‌گري‌هاي مالي و پولي',
    },
    {
        'code': '66',
        'name': 'بيمه وصندوق بازنشستگي به جزتامين اجتماعي',
    },
    {
        'code': '67',
        'name': 'فعاليتهاي كمكي به نهادهاي مالي واسط',
    },
    {
        'code': '68',
        'name': 'صندوق سرمايه گذاري قابل معامله',
    },
    {
        'code': '69',
        'name': 'اوراق تامين مالي',
    },
    {
        'code': '70',
        'name': 'انبوه سازي، املاك و مستغلات',
    },
    {
        'code': '71',
        'name': 'فعاليت مهندسي، تجزيه، تحليل و آزمايش فني',
    },
    {
        'code': '72',
        'name': 'رايانه و فعاليت‌هاي وابسته به آن',
    },
    {
        'code': '73',
        'name': 'اطلاعات و ارتباطات',
    },
    {
        'code': '74',
        'name': 'خدمات فني و مهندسي',
    },
    {
        'code': '76',
        'name': 'اوراق بهادار مبتني بر دارايي فكري',
    },
    {
        'code': '77',
        'name': 'فعالبت هاي اجاره و ليزينگ',
    },
    {
        'code': '82',
        'name': 'فعاليت پشتيباني اجرائي اداري وحمايت كسب',
    },
    {
        'code': '90',
        'name': 'فعاليت هاي هنري، سرگرمي و خلاقانه',
    },
    {
        'code': '93',
        'name': 'فعاليتهاي فرهنگي و ورزشي',
    },
    {
        'code': '98',
        'name': 'گروه اوراق غيرفعال',
    },
    {
        'code': 'X1',
        'name': 'شاخص',
    },
]


def get_all_groups():
    return _GROUPS
